@@ -33,11 +33,11 @@ Geo::Coder::Free::Maxmind - Provides a geocoding functionality using the MaxMind
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 =head1 SYNOPSIS
 
@@ -131,7 +131,8 @@ sub geocode {
 		return Geo::Location::Point->new({
 			'lat' => $rc->{'latitude'},
 			'long' => $rc->{'longitude'},
-			'location' => $location
+			'location' => $location,
+			'database' => 'Maxmind'
 		});
 	}
 
@@ -199,6 +200,9 @@ sub geocode {
 		}
 		my $countrycode = country2code($country);
 		# ::diag(__LINE__, ": country $countrycode, county $county, state $state, location $location");
+		# if($county && $countrycode) {
+			# ::diag(__LINE__, ": country $countrycode, county $county, location $location");
+		# }
 
 		if($state && $admin1cache{$state}) {
 			$concatenated_codes = $admin1cache{$state};
@@ -234,8 +238,8 @@ sub geocode {
 			}
 		}
 	}
-	# ::diag(__LINE__, ": $concatenated_codes");
 	return unless(defined($concatenated_codes));
+	# ::diag(__LINE__, ": $concatenated_codes");
 
 	my @admin2s;
 	my $region;
@@ -259,56 +263,55 @@ sub geocode {
 	if(defined($county) && ($county =~ /^[A-Z]{2}$/) && ($country =~ /^(United States|USA|US)$/)) {
 		# US state. Not Canadian province.
 		$region = $county;
+	} elsif($county && $admin1cache{$county}) {
+		# ::diag(__LINE__);
+		$region = $admin1cache{$county};
+	} elsif($county && $admin2cache{$county}) {
+		$region = $admin2cache{$county};
+		# ::diag(__LINE__, ": $county");
+	} elsif(defined($state) && $admin2cache{$state} && !defined($county)) {
+		# ::diag(__LINE__);
+		$region = $admin2cache{$state};
 	} else {
-		if($county && $admin1cache{$county}) {
 		# ::diag(__LINE__);
-			$region = $admin1cache{$county};
-		} elsif($county && $admin2cache{$county}) {
-		# ::diag(__LINE__);
-			$region = $admin2cache{$county};
-		} elsif(defined($state) && $admin2cache{$state} && !defined($county)) {
-		# ::diag(__LINE__);
-			$region = $admin2cache{$state};
+		if(defined($county) && ($county eq 'London')) {
+			@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $location);
 		} else {
-		# ::diag(__LINE__);
-			if(defined($county) && ($county eq 'London')) {
-				@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $location);
-			} else {
-			# ::diag(__LINE__, ": $county");
-				@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $county);
-			}
-			foreach my $admin2(@admin2s) {
-				# ::diag(__LINE__, Data::Dumper->new([$admin2])->Dump());
-				if($admin2->{'concatenated_codes'} =~ $concatenated_codes) {
-					$region = $admin2->{'concatenated_codes'};
-					if($region =~ /^[A-Z]{2}\.([A-Z]{2})\./) {
-						my $rc = $1;
-						if(defined($state) && ($state =~ /^[A-Z]{2}$/)) {
-							if($state eq $rc) {
-								$region = $rc;
-								@regions = ();
-								last;
-							}
-						} else {
-							push @regions, $region;
-							push @regions, $rc;
+		# ::diag(__LINE__, ": $county");
+			@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $county);
+		}
+		# ::diag(__LINE__, Data::Dumper->new([\@admin2s])->Dump());
+		foreach my $admin2(@admin2s) {
+			# ::diag(__LINE__, Data::Dumper->new([$admin2])->Dump());
+			if($admin2->{'concatenated_codes'} =~ $concatenated_codes) {
+				$region = $admin2->{'concatenated_codes'};
+				if($region =~ /^[A-Z]{2}\.([A-Z]{2})\./) {
+					my $rc = $1;
+					if(defined($state) && ($state =~ /^[A-Z]{2}$/)) {
+						if($state eq $rc) {
+							$region = $rc;
+							@regions = ();
+							last;
 						}
 					} else {
 						push @regions, $region;
+						push @regions, $rc;
 					}
+				} else {
+					push @regions, $region;
 				}
 			}
-			if($state && !defined($region)) {
-				if($state =~ /^[A-Z]{2}$/) {
-					$region = $state;
-					@regions = ();
-				} else {
-					@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $state);
-					foreach my $admin2(@admin2s) {
-						if($admin2->{'concatenated_codes'} =~ $concatenated_codes) {
-							$region = $admin2->{'concatenated_codes'};
-							last;
-						}
+		}
+		if($state && !defined($region)) {
+			if($state =~ /^[A-Z]{2}$/) {
+				$region = $state;
+				@regions = ();
+			} else {
+				@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $state);
+				foreach my $admin2(@admin2s) {
+					if($admin2->{'concatenated_codes'} =~ $concatenated_codes) {
+						$region = $admin2->{'concatenated_codes'};
+						last;
 					}
 				}
 			}
@@ -318,7 +321,7 @@ sub geocode {
 	if((scalar(@regions) == 0) && !defined($region)) {
 		# e.g. Unitary authorities in the UK
 		# admin[12].db columns are labelled ['concatenated_codes', 'name', 'asciiname', 'geonameId']
-	# 	# ::diag(__LINE__, ": $location");
+		# ::diag(__LINE__, ": $location");
 		@admin2s = $self->{'admin2'}->selectall_hash(asciiname => $location);
 		if(scalar(@admin2s) && defined($admin2s[0]->{'concatenated_codes'})) {
 			foreach my $admin2(@admin2s) {
@@ -381,7 +384,7 @@ sub geocode {
 		$options->{'Country'} = lc($c);
 		$confidence = 0.1;
 	}
-	# ::diag(__LINE__, Data::Dumper->new([$options])->Dump());
+	# ::diag(__LINE__, ': ', Data::Dumper->new([$options])->Dump());
 	# This case nonsense is because DBD::CSV changes the columns to lowercase, wherease DBD::SQLite does not
 	if(wantarray) {
 		my @rc = $self->{'cities'}->selectall_hash($options);
@@ -392,7 +395,7 @@ sub geocode {
 				return;
 			}
 		}
-	 	# ::diag(__LINE__, Data::Dumper->new([\@rc])->Dump());
+	 	# ::diag(__LINE__, ': ', Data::Dumper->new([\@rc])->Dump());
 		foreach my $city(@rc) {
 			if($city->{'Latitude'}) {
 				$city->{'latitude'} = delete $city->{'Latitude'};
@@ -432,14 +435,17 @@ sub geocode {
 			push @locations, Geo::Location::Point->new({
 				'lat' => $l->{'latitude'},
 				'long' => $l->{'longitude'},
-				'location' => $location
+				'location' => $location,
+				'database' => 'Maxmind'
 			});
 		}
 
 		return @locations;
 	}
+	# ::diag(__LINE__, ': ', Data::Dumper->new([$options])->Dump());
 	my $city = $self->{'cities'}->fetchrow_hashref($options);
 	if(!defined($city)) {
+		# ::diag(__LINE__, ': ', scalar(@regions));
 		foreach $region(@regions) {
 			if($region =~ /^.+\.(.+)$/) {
 				$region = $1;
@@ -453,13 +459,13 @@ sub geocode {
 		}
 	}
 
-	# ::diag(__LINE__, Data::Dumper->new([$city])->Dump());
+	# ::diag(__LINE__, ': ', Data::Dumper->new([$city])->Dump());
 	if(defined($city) && defined($city->{'Latitude'})) {
 		$city->{'confidence'} = $confidence;
 		return Geo::Location::Point->new($city);
 	}
 	# return $city;
-	undef;
+	return;
 }
 
 =head2	reverse_geocode
@@ -484,8 +490,7 @@ sub reverse_geocode {
 		$param{'latlng'} = shift;
 	}
 
-	my $latlng = $param{'latlng'}
-		or Carp::croak('Usage: reverse_geocode(latlng => $location)');
+	my $latlng = $param{'latlng'};
 
 	my $latitude;
 	my $longitude;
@@ -498,6 +503,10 @@ sub reverse_geocode {
 		$longitude //= $param{'long'};
 	}
 
+	if((!defined($latitude)) || !defined($longitude)) {
+		Carp::croak('Usage: reverse_geocode(latlng => $location)');
+	}
+
 	if(!defined($self->{'cities'})) {
 		$self->{'cities'} = Geo::Coder::Free::DB::MaxMind::cities->new(
 			cache => $self->{cache} || CHI->new(driver => 'Memory', datastore => {})
@@ -506,64 +515,46 @@ sub reverse_geocode {
 
 	if(wantarray) {
 		my @locs = $self->{'cities'}->execute("SELECT * FROM cities WHERE (ABS(Latitude - $latitude) < 0.01) AND (ABS(Longitude - $longitude) < 0.01)");
-		# Change 'Charing Cross, P5, Gb' to 'Charing Cross, London, Gb'
 		foreach my $loc(@locs) {
-			if(my $region = $loc->{'Region'}) {
-				my $county;
-				foreach $county(keys %admin2cache) {
-					if($admin2cache{$county} eq $region) {
-						last;
-					}
-				}
-				if($county) {
-					$loc->{'Region'} = $county;
-				} else {
-					$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
-					my $row = $self->{'admin2'}->execute("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($loc->{'Country'}) . '.%.' . uc($region) . "'");
-					if(ref($row) && $row->{'name'}) {
-						$admin2cache{$row->{'name'}} = $region;
-						$loc->{'Region'} = $row->{'name'};
-					}
-				}
-			}
+			$self->_prepare($loc);
 		}
-		# ::diag(Data::Dumper->new([\@locs])->Dump());
-		# my @rc;
-		# foreach my $loc(@locs) {
-			# push @rc, Geo::Location::Point->new($loc)->as_string();
-		# }
-		# return @rc;
 		return map { Geo::Location::Point->new($_)->as_string() } @locs;
 	}
-	# TODO - this is similar to the wantarray code, just the LIMIT 1 and
-	#	no map { } code.  Need to combine
-	if(my $rc = $self->{'cities'}->execute("SELECT * FROM cities WHERE (ABS(Latitude - $latitude) < 0.01) AND (ABS(Longitude - $longitude) < 0.01) LIMIT 1")) {
-		if(my $region = $rc->{'Region'}) {
-			my $county;
-			foreach $county(keys %admin2cache) {
-				if($admin2cache{$county} eq $region) {
-					last;
-				}
-			}
-			if($county) {
-				$rc->{'Region'} = $county;
-			} else {
-				$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
-				# ::diag("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($rc->{'Country'}) . '.%.' . uc($region) . "' LIMIT 1");
-				my $row = $self->{'admin2'}->execute("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($rc->{'Country'}) . '.%.' . uc($region) . "' LIMIT 1");
-				# print Data::Dumper->new([$row])->Dump();
-				# ::diag(Data::Dumper->new([$row])->Dump());
-				if(ref($row) && $row->{'name'}) {
-					$admin2cache{$row->{'name'}} = $region;
-					$rc->{'Region'} = $row->{'name'};
-				}
-			}
+	# Try close in then zoom out, to a reasonable limit
+	foreach my $radius(0.000001, 0.00001, 0.0001, 0.001, 0.01) {
+		if(my $rc = $self->{'cities'}->execute("SELECT * FROM cities WHERE (ABS(Latitude - $latitude) < $radius) AND (ABS(Longitude - $longitude) < $radius) LIMIT 1")) {
+			$self->_prepare($rc);
+			return Geo::Location::Point->new($rc)->as_string();
 		}
-		return Geo::Location::Point->new($rc)->as_string();
 	}
 	return;
 }
 
+# Change 'Charing Cross, P5, Gb' to 'Charing Cross, London, Gb'
+sub _prepare {
+	my ($self, $loc) = @_;
+
+	if(my $region = $loc->{'Region'}) {
+		my $county;
+		foreach my $c(keys %admin2cache) {
+			if($admin2cache{$c} eq $region) {
+				$county = $c;
+				last;
+			}
+		}
+		if($county) {
+			$loc->{'Region'} = $county;
+		} else {
+			$self->{'admin2'} //= Geo::Coder::Free::DB::MaxMind::admin2->new() or die "Can't open the admin2 database";
+			my $row = $self->{'admin2'}->execute("SELECT name FROM admin2 WHERE concatenated_codes LIKE '" . uc($loc->{'Country'}) . '.%.' . uc($region) . "' LIMIT 1");
+			if(ref($row) && $row->{'name'}) {
+				$admin2cache{$row->{'name'}} = $region;
+				$loc->{'Region'} = $row->{'name'};
+			}
+		}
+	}
+}
+
 =head2	ua
 
 Does nothing, here for compatibility with other geocoders
@@ -592,6 +583,11 @@ The database contains Canadian cities, but not provinces, so a search for "New B
 
 The GeoNames admin databases are in this class, they should be in Geo::Coder::GeoNames.
 
+The data at
+L<https://github.com/apache/commons-csv/blob/master/src/test/resources/perf/worldcitiespop.txt.gz?raw=true>
+are 7 years out of date,
+and are unconsistent with the Geonames database.
+
 =head1 SEE ALSO
 
 VWF, MaxMind and geonames.
